@@ -11,6 +11,8 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  */
+
+#include <stdlib.h> /* We use atoi() here.. */
 #include "xmms/i18n.h"
 #include "ir.h"
 
@@ -18,6 +20,7 @@ static gboolean keepConfGoing;
 gboolean irconf_is_going = FALSE;
 static GtkWidget *irconf_mainwin, *irconf_controlwin, *irconf_playlistwin;
 static GtkWidget *dev_entry, *ircode_entry, *playlist_entry, *playlist_spin;
+static GtkWidget *codelen_entry;
 
 static gchar *ir_control[13] =
 {
@@ -39,6 +42,14 @@ static gint ir_was_enabled;
 static void irconf_ok_cb(GtkWidget * w, gpointer data)
 {
 	ircfg.device = g_strdup(gtk_entry_get_text(GTK_ENTRY(dev_entry)));
+	ircfg.codelen = atoi(gtk_entry_get_text(GTK_ENTRY(codelen_entry)));
+	if(ircfg.codelen > IR_MAX_CODE_LEN)
+	    ircfg.codelen = IR_MAX_CODE_LEN;
+	if(ircfg.codelen < 0)
+	    ircfg.codelen = 0;
+	/* Re-initialize remote using new settings */
+	ir_close_port();
+	ir_open_port(ircfg.device);
 	irapp_save_config();
 	gtk_widget_destroy(irconf_mainwin);
 }
@@ -191,7 +202,7 @@ static void irconf_control_cb(GtkWidget * w, gchar * data)
 		gtk_button_box_set_spacing(GTK_BUTTON_BOX(box), 5);
 		gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 0);
 
-		button = gtk_button_new_with_label(_("Ok"));
+		button = gtk_button_new_with_label(_("OK"));
 		gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(irconf_control_ok_cb), NULL);
 		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 		gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
@@ -231,7 +242,7 @@ static void pl_entry_change_cb(GtkWidget * widget, GtkSpinButton * spin)
 void ir_configure(void)
 {
 	GtkWidget *vbox, *notebook, *box, *frame, *table, *vbox2, *label,
-	         *button;
+	         *button, *label_codelen;
 	GtkAdjustment *adj;
 	gint i;
 
@@ -273,6 +284,17 @@ void ir_configure(void)
 		gtk_entry_set_text(GTK_ENTRY(dev_entry), ircfg.device);
 		gtk_table_attach_defaults(GTK_TABLE(table), dev_entry, 1, 2, 0, 1);
 		gtk_widget_show(dev_entry);
+
+                label_codelen = gtk_label_new(_("IR code length: "));
+		gtk_misc_set_alignment(GTK_MISC(label_codelen), 1.0, 0.5);
+		gtk_table_attach_defaults(GTK_TABLE(table), label_codelen, 0, 1, 1, 2);
+		gtk_widget_show(label_codelen);
+
+		codelen_entry = gtk_entry_new();
+		gtk_entry_set_text(GTK_ENTRY(codelen_entry), g_strdup_printf("%d", ircfg.codelen));
+		gtk_table_attach_defaults(GTK_TABLE(table), codelen_entry, 1, 2, 1, 2);
+		gtk_widget_show(codelen_entry);
+
 		gtk_widget_show(table);
 		gtk_widget_show(frame);
 
@@ -280,7 +302,6 @@ void ir_configure(void)
 		gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
 
 		vbox2 = gtk_vbox_new(FALSE, 0);
-//              gtk_container_set_border_width(GTK_CONTAINER(vbox2),0);
 		gtk_container_add(GTK_CONTAINER(frame), vbox2);
 
 		table = gtk_table_new(5, 3, TRUE);
@@ -352,7 +373,7 @@ void ir_configure(void)
 		gtk_button_box_set_spacing(GTK_BUTTON_BOX(box), 5);
 		gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 0);
 
-		button = gtk_button_new_with_label(_("Ok"));
+		button = gtk_button_new_with_label(_("OK"));
 		gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(irconf_ok_cb), NULL);
 		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 		gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
