@@ -40,6 +40,7 @@
 typedef struct
 {
 	gchar *device;
+	gint codelen;
 	gchar *button_play, *button_stop, *button_next, *button_prev;
 	gchar *button_pause, *button_seekf, *button_seekb;
 	gchar *button_volup, *button_voldown, *button_plus100;
@@ -48,14 +49,13 @@ typedef struct
 }
 irConfig;
 
-extern pthread_t irapp_thread;
-extern gboolean keepGoing;
 extern irConfig ircfg;
 extern gboolean irconf_is_going;
 
 
 void ir_about(void);
 void ir_configure(void);
+
 void irapp_read_config(void);
 void irapp_save_config(void);
 void irapp_init_port(gchar * ir_port);
