@@ -321,8 +321,18 @@ static void configurewin_check_drive(GtkButton *w, gpointer data)
 				_("Error: %s exists, but is not a directory"),
 				directory);
 		else
-			g_string_sprintfa(str, _("Directory %s OK."),
-					  directory);
+		{
+			if (!access(directory, R_OK))
+				g_string_sprintfa(str, _("Directory %s OK."),
+						  directory);
+			else
+				g_string_sprintfa(str,
+						  _("Directory %s exists, but "
+						    "you do not have permission "
+						    "to access it."),
+						  directory);
+		}
+				
 	}
 			
 	window = gtk_window_new(GTK_WINDOW_DIALOG);
@@ -660,7 +670,7 @@ void cdda_configure(void)
 	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
 	gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
 
-	ok = gtk_button_new_with_label(_("Ok"));
+	ok = gtk_button_new_with_label(_("OK"));
 	gtk_signal_connect(GTK_OBJECT(ok), "clicked",
 			   GTK_SIGNAL_FUNC(cdda_configurewin_ok_cb), NULL);
 	gtk_signal_connect(GTK_OBJECT(ok), "clicked",
