@@ -1,5 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
+ *                           Thomas Nilsson and 4Front Technologies
+ *  Copyright (C) 1999-2005  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,20 +24,52 @@
 #define NFRAGS		32
 
 static gint fd = 0;
-static gpointer buffer;
-static gboolean going = FALSE, prebuffer, paused = FALSE, unpause = FALSE,
-         do_pause = FALSE, remove_prebuffer = FALSE;
+static char *buffer;
+static gboolean going, prebuffer, paused, unpause, do_pause, remove_prebuffer;
 static gint device_buffer_used, buffer_size, prebuffer_size, blk_size;
 static gint rd_index = 0, wr_index = 0;
 static gint output_time_offset = 0;
 static guint64 written = 0, output_bytes = 0;
-static gint bps, ebps;
 static gint flush;
-static gint fragsize, oss_format, format, channels, frequency, efrequency, device_buffer_size;
-static gint input_bps, input_format, input_frequency, input_channels;
+static gint fragsize, device_buffer_size;
 static gchar *device_name;
 static pthread_t buffer_thread;
-static gboolean realtime = FALSE, select_works = TRUE;
+static gboolean realtime, select_works;
+
+static int (*oss_convert_func)(void **data, int length);
+static int (*oss_stereo_convert_func)(void **data, int length, int fmt);
+
+struct format_info {
+	union {
+		AFormat xmms;
+		int oss;
+	} format;
+	int frequency;
+	int channels;
+	int bps;
+};
+
+
+/*
+ * The format of the data from the input plugin
+ * This will never change during a song. 
+ */
+struct format_info input;
+
+/*
+ * The format we get from the effect plugin.
+ * This will be different from input if the effect plugin does
+ * some kind of format conversion.
+ */
+struct format_info effect;
+
+/*
+ * The format of the data we actually send to the soundcard.
+ * This might be different from effect if we need to resample or do
+ * some other format conversion.
+ */
+struct format_info output;
+
 
 static void oss_calc_device_buffer_used(void)
 {
@@ -47,66 +81,85 @@ static void oss_calc_device_buffer_used(void)
 }
 
 
-gint oss_downsample(gpointer ob, guint length, guint speed, guint espeed);
+static gint oss_downsample(gpointer ob, guint length, guint speed, guint espeed);
 
-static void oss_setup_format(AFormat fmt,gint rate, gint nch)
+static int oss_calc_bitrate(int oss_fmt, int rate, int channels)
 {
-	format = fmt;
-	frequency = rate;
-	channels = nch;
+	int bitrate = rate * channels;
+
+	if (oss_fmt == AFMT_U16_BE || oss_fmt == AFMT_U16_LE ||
+	    oss_fmt == AFMT_S16_BE || oss_fmt == AFMT_S16_LE)
+		bitrate *= 2;
+
+	return bitrate;
+}
+
+static int oss_get_format(AFormat fmt)
+{
+	int format = 0;
+
 	switch (fmt)
 	{
 		case FMT_U8:
-			oss_format = AFMT_U8;
+			format = AFMT_U8;
 			break;
 		case FMT_S8:
-			oss_format = AFMT_S8;
+			format = AFMT_S8;
 			break;
 		case FMT_U16_LE:
-			oss_format = AFMT_U16_LE;
+			format = AFMT_U16_LE;
 			break;
 		case FMT_U16_BE:
-			oss_format = AFMT_U16_BE;
+			format = AFMT_U16_BE;
 			break;
 		case FMT_U16_NE:
-#ifdef AFMT_U16_NE
-			oss_format = AFMT_U16_NE;
-#else
 #ifdef WORDS_BIGENDIAN
-			oss_format = AFMT_U16_BE;
+			format = AFMT_U16_BE;
 #else
-			oss_format = AFMT_U16_LE;
-#endif
+			format = AFMT_U16_LE;
 #endif
 			break;
 		case FMT_S16_LE:
-			oss_format = AFMT_S16_LE;
+			format = AFMT_S16_LE;
 			break;
 		case FMT_S16_BE:
-			oss_format = AFMT_S16_BE;
+			format = AFMT_S16_BE;
 			break;
 		case FMT_S16_NE:
-#ifdef AFMT_S16_NE
-			oss_format = AFMT_S16_NE;
-#else
 #ifdef WORDS_BIGENDIAN
-			oss_format = AFMT_S16_BE;
+			format = AFMT_S16_BE;
 #else
-			oss_format = AFMT_S16_LE;
-#endif
+			format = AFMT_S16_LE;
 #endif
 			break;
 	}
 
-	bps = rate * nch;
-	if (oss_format == AFMT_U16_BE || oss_format == AFMT_U16_LE || oss_format == AFMT_S16_BE || oss_format == AFMT_S16_LE)
-		bps *= 2;
+	return format;
+}
+
+static void oss_setup_format(AFormat fmt, int rate, int nch)
+{
+	effect.format.xmms = fmt;
+	effect.frequency = rate;
+	effect.channels = nch;
+	effect.bps = oss_calc_bitrate(oss_get_format(fmt), rate, nch);
+
+	output.format.oss = oss_get_format(fmt);
+	output.frequency = rate;
+	output.channels = nch;
+
+
 	fragsize = 0;
-	while ((1L << fragsize) < bps / 25)
+	while ((1L << fragsize) < effect.bps / 25)
 		fragsize++;
 	fragsize--;
 
 	device_buffer_size = ((1L << fragsize) * (NFRAGS + 1));
+
+	oss_set_audio_params();
+
+	output.bps = oss_calc_bitrate(output.format.oss, output.frequency,
+				      output.channels);
 }
 	
 
@@ -114,25 +167,25 @@ gint oss_get_written_time(void)
 {
 	if (!going)
 		return 0;
-	return (gint) ((written * 1000) / input_bps);
+	return (written * 1000) / effect.bps;
 }
 
 gint oss_get_output_time(void)
 {
-	
 	guint64 bytes;
 
 	if (!fd || !going)
 		return 0;
 
-	if(realtime)
+	if (realtime)
 		oss_calc_device_buffer_used();
-	bytes = output_bytes < device_buffer_used ? 0 : output_bytes - device_buffer_used;
+	bytes = output_bytes < device_buffer_used ?
+		0 : output_bytes - device_buffer_used;
 
-	return output_time_offset + (gint) ((bytes  * 1000) / ebps);
+	return output_time_offset + ((bytes * 1000) / output.bps);
 }
 
-gint oss_used(void)
+static int oss_used(void)
 {
 	if (realtime)
 		return 0;
@@ -179,16 +232,34 @@ gint oss_free(void)
 			return 1000000;
 }
 
-static void oss_write_audio(gpointer data,gint length)
+static inline ssize_t write_all(int fd, const void *buf, size_t count)
+{
+	ssize_t done = 0;
+	do {
+		ssize_t n = write(fd, (char *)buf + done, count - done);
+		if (n == -1)
+		{
+			if (errno == EINTR)
+				continue;
+			else
+				break;
+		}
+		done += n;
+	} while (count > done);
+
+	return done;
+}
+
+static void oss_write_audio(gpointer data, int length)
 {
 	audio_buf_info abuf_info;
 	AFormat new_format;
-	gint new_frequency,new_channels;
+	int new_frequency, new_channels;
 	EffectPlugin *ep;
 	
-	new_format = input_format;
-	new_frequency = input_frequency;
-	new_channels = input_channels;
+	new_format = input.format.xmms;
+	new_frequency = input.frequency;
+	new_channels = input.channels;
 	
 	ep = get_current_effect_plugin();
 	if(effects_enabled() && ep && ep->query_format)
@@ -196,42 +267,60 @@ static void oss_write_audio(gpointer data,gint length)
 		ep->query_format(&new_format,&new_frequency,&new_channels);
 	}
 	
-	if(new_format != format || new_frequency != frequency || new_channels != channels)
+	if (new_format != effect.format.xmms ||
+	    new_frequency != effect.frequency ||
+	    new_channels != effect.channels)
 	{
-		output_time_offset += (gint) ((output_bytes * 1000) / ebps);
+		output_time_offset += (output_bytes * 1000) / output.bps;
 		output_bytes = 0;
-		oss_setup_format(new_format, new_frequency, new_channels);
-		frequency = new_frequency;
-		channels = new_channels;
 		close(fd);
 		fd = open(device_name,O_WRONLY);
-		oss_set_audio_params();
+		oss_setup_format(new_format, new_frequency, new_channels);
 	}
-	if(effects_enabled() && ep && ep->mod_samples)
-		length = ep->mod_samples(&data,length, input_format, input_frequency, input_channels);
-	if(realtime && !ioctl(fd,SNDCTL_DSP_GETOSPACE,&abuf_info))
+	if (effects_enabled() && ep && ep->mod_samples)
+		length = ep->mod_samples(&data, length,
+					 input.format.xmms,
+					 input.frequency,
+					 input.channels);
+	if (realtime && !ioctl(fd, SNDCTL_DSP_GETOSPACE, &abuf_info))
 	{
-		while(abuf_info.bytes < length)
+		while (abuf_info.bytes < length)
 		{
 			xmms_usleep(10000);
-			if(ioctl(fd,SNDCTL_DSP_GETOSPACE,&abuf_info))
+			if (ioctl(fd, SNDCTL_DSP_GETOSPACE, &abuf_info))
 				break;
 		}
 	}
-	if(frequency == efrequency)
-		output_bytes += write(fd,data,length);
+
+	if (oss_convert_func != NULL)
+		length = oss_convert_func(&data, length);
+
+	if (oss_stereo_convert_func != NULL)
+		length = oss_stereo_convert_func(&data, length,
+						 output.format.oss);
+
+	if (effect.frequency == output.frequency)
+		output_bytes += write_all(fd, data, length);
 	else
-		output_bytes += oss_downsample(data,length,frequency,efrequency);
+		output_bytes += oss_downsample(data, length,
+					       effect.frequency,
+					       output.frequency);
 }
 
-static void swap_words(guint16 *buffer, gint length)
+static void swap_endian(guint16 *data, int length)
 {
-	guint16 *ptr = buffer;
-	gint i;
-	for(i = 0; i < length; i++, ptr++)
-		*ptr = ((*ptr & 0x00FF) << 8) | (*ptr >> 8);
+	int i;
+	for (i = 0; i < length; i += 2, data++)
+		*data = GUINT16_SWAP_LE_BE(*data);
 }
 
+#define NOT_NATIVE_ENDIAN ((IS_BIG_ENDIAN &&				\
+			   (output.format.oss == AFMT_S16_LE ||		\
+			    output.format.oss == AFMT_U16_LE)) ||	\
+			  (!IS_BIG_ENDIAN &&				\
+			   (output.format.oss == AFMT_S16_BE ||		\
+			    output.format.oss == AFMT_U16_BE)))
+
 
 #define RESAMPLE_STEREO(sample_type)				\
 do {								\
@@ -242,13 +331,8 @@ do {								\
 	if (nlen == 0)						\
 		break;						\
 	nlen <<= shift;						\
-	if((big_endian &&					\
-	    (oss_format == AFMT_S16_LE ||			\
-	     oss_format == AFMT_U16_LE)) ||			\
-	   (!big_endian &&					\
-	    (oss_format == AFMT_S16_BE ||			\
-	     oss_format == AFMT_U16_BE)))			\
-		swap_words(ob, length >> 1);			\
+	if (NOT_NATIVE_ENDIAN)					\
+		swap_endian(ob, length);			\
 	if(nlen > nbuffer_size)					\
 	{							\
 		nbuffer = g_realloc(nbuffer, nlen);		\
@@ -277,14 +361,9 @@ do {								\
 			  frac) >> 12);				\
 		x += delta;					\
 	}							\
-	if ((big_endian &&					\
-	     (oss_format == AFMT_S16_LE ||			\
-	      oss_format == AFMT_U16_LE)) ||			\
-	    (!big_endian &&					\
-	     (oss_format == AFMT_S16_BE ||			\
-	      oss_format == AFMT_U16_BE)))			\
-		swap_words(nbuffer, nlen >> 1);			\
-	w = write(fd, nbuffer, nlen);				\
+	if (NOT_NATIVE_ENDIAN)					\
+		swap_endian(nbuffer, nlen);			\
+	w = write_all(fd, nbuffer, nlen);			\
 } while (0)
 
 
@@ -297,13 +376,8 @@ do {								\
 	if (nlen == 0)						\
 		break;						\
 	nlen <<= shift;						\
-	if((big_endian &&					\
-	    (oss_format == AFMT_S16_LE ||			\
-	     oss_format == AFMT_U16_LE)) ||			\
-	   (!big_endian &&					\
-	    (oss_format == AFMT_S16_BE ||			\
-	     oss_format == AFMT_U16_BE)))			\
-		swap_words(ob, length >> 1);			\
+	if (NOT_NATIVE_ENDIAN)					\
+		swap_endian(ob, length);			\
 	if(nlen > nbuffer_size)					\
 	{							\
 		nbuffer = g_realloc(nbuffer, nlen);		\
@@ -324,52 +398,42 @@ do {								\
 			  inptr[(x1 >> 12) + 1] * frac) >> 12);	\
 		x += delta;					\
 	}							\
-	if ((big_endian &&					\
-	    (oss_format == AFMT_S16_LE ||			\
-	     oss_format == AFMT_U16_LE)) ||			\
-	   (!big_endian &&					\
-	    (oss_format == AFMT_S16_BE ||			\
-	     oss_format == AFMT_U16_BE)))			\
-		swap_words(nbuffer, nlen >> 1);			\
-	w = write(fd, nbuffer, nlen);				\
+	if (NOT_NATIVE_ENDIAN)					\
+		swap_endian(nbuffer, nlen);			\
+	w = write_all(fd, nbuffer, nlen);			\
 } while (0)
 
 
-gint oss_downsample(gpointer ob, guint length, guint speed, guint espeed)
+static gint oss_downsample(gpointer ob, guint length, guint speed, guint espeed)
 {
 	guint w = 0;
 	static gpointer nbuffer = NULL;
 	static gint nbuffer_size = 0;
-#ifdef WORDS_BIGENDIAN
-	const gboolean big_endian = TRUE;
-#else
-	const gboolean big_endian = FALSE;
-#endif
 
-	switch (oss_format)
+	switch (output.format.oss)
 	{
 		case AFMT_S16_BE:
 		case AFMT_S16_LE:
-			if(channels == 2)
+			if (output.channels == 2)
 				RESAMPLE_STEREO(gint16);
 			else
 				RESAMPLE_MONO(gint16);
 			break;
 		case AFMT_U16_BE:
 		case AFMT_U16_LE:
-			if(channels == 2)
+			if (output.channels == 2)
 				RESAMPLE_STEREO(guint16);
 			else
 				RESAMPLE_MONO(guint16);
 			break;
 		case AFMT_S8:
-			if(channels == 2)
+			if (output.channels == 2)
 				RESAMPLE_STEREO(gint8);
 			else
 				RESAMPLE_MONO(gint8);
 			break;
 		case AFMT_U8:
-			if(channels == 2)
+			if (output.channels == 2)
 				RESAMPLE_STEREO(guint8);
 			else
 				RESAMPLE_MONO(guint8);
@@ -378,9 +442,9 @@ gint oss_downsample(gpointer ob, guint length, guint speed, guint espeed)
 	return w;
 }
 
-void oss_write(gpointer ptr, gint length)
+void oss_write(gpointer ptr, int length)
 {
-	gint cnt, off = 0;
+	int cnt, off = 0;
 
 	if (!realtime)
 	{
@@ -390,22 +454,19 @@ void oss_write(gpointer ptr, gint length)
 		while (length > 0)
 		{
 			cnt = MIN(length, buffer_size - wr_index);
-			memcpy((gchar *)buffer + wr_index, (gchar *)ptr + off, cnt);
+			memcpy(buffer + wr_index, (char *)ptr + off, cnt);
 			wr_index = (wr_index + cnt) % buffer_size;
 			length -= cnt;
 			off += cnt;
-
 		}
 	}
 	else
 	{
 		if (paused)
 			return;
-		oss_write_audio(ptr,length);
+		oss_write_audio(ptr, length);
 		written += length;
-		
 	}
-
 }
 
 void oss_close(void)
@@ -421,6 +482,7 @@ void oss_close(void)
 		close(fd);
 	}
 	g_free(device_name);
+	oss_free_convert_buffer();
 	wr_index = 0;
 	rd_index = 0;
 }
@@ -440,7 +502,7 @@ void oss_flush(gint time)
 		fd = open(device_name, O_WRONLY);
 		oss_set_audio_params();
 		output_time_offset = time;
-		written = (guint64)(time / 10) * (guint64)(input_bps / 100);		
+		written = ((guint64)time * input.bps) / 1000;
 		output_bytes = 0;
 	}
 }
@@ -481,33 +543,29 @@ void *oss_loop(void *arg)
 				while (length > 0)
 				{
 					cnt = MIN(length,buffer_size-rd_index);
-					oss_write_audio((gchar *)buffer + rd_index, cnt);
+					oss_write_audio(buffer + rd_index, cnt);
 					rd_index=(rd_index+cnt)%buffer_size;
 					length-=cnt;				
 				}
 				if (!oss_used())
-				{
 					ioctl(fd, SNDCTL_DSP_POST, 0);
-				}
 			}
 		}
 		else
-		{
 			xmms_usleep(10000);
-		}
 		oss_calc_device_buffer_used();
 		if (do_pause && !paused)
 		{
 			do_pause = FALSE;
 			paused = TRUE;
-			rd_index -= device_buffer_used;
-			output_bytes -= device_buffer_used;
-			if (rd_index < 0)
-				rd_index += buffer_size;
+			/*
+			 * We lose some data here that is sent to the
+			 * soundcard, but not yet played.  I don't
+			 * think this is worth fixing.
+			 */
 			ioctl(fd, SNDCTL_DSP_RESET, 0);
-
 		}
-		if (unpause && paused)
+		else if (unpause && paused)
 		{
 			unpause = FALSE;
 			close(fd);
@@ -519,8 +577,9 @@ void *oss_loop(void *arg)
 		if (flush != -1)
 		{
 			/*
-			 * This close and open is a work around of a bug that exists in some drivers which 
-			 * cause the driver to get fucked up by a reset
+			 * This close and open is a work around of a
+			 * bug that exists in some drivers which cause
+			 * the driver to get fucked up by a reset
 			 */
 
 			ioctl(fd, SNDCTL_DSP_RESET, 0);
@@ -528,7 +587,7 @@ void *oss_loop(void *arg)
 			fd = open(device_name, O_WRONLY);
 			oss_set_audio_params();
 			output_time_offset = flush;
-			written = (guint64)(flush / 10) * (guint64)(input_bps / 100);
+			written = ((guint64)flush * input.bps) / 1000;
 			rd_index = wr_index = output_bytes = 0;
 			flush = -1;
 			prebuffer = TRUE;
@@ -544,38 +603,51 @@ void *oss_loop(void *arg)
 
 void oss_set_audio_params(void)
 {
-	gint frag, stereo, ret;
+	int frag, stereo, ret;
 	struct timeval tv;
 	fd_set set;
-	
+
 	ioctl(fd, SNDCTL_DSP_RESET, 0);
 	frag = (NFRAGS << 16) | fragsize;
 	ioctl(fd, SNDCTL_DSP_SETFRAGMENT, &frag);
-	ioctl(fd, SNDCTL_DSP_SETFMT, &oss_format);
-	ioctl(fd, SNDCTL_DSP_SETFMT, &oss_format);
-	stereo = channels - 1;
-	ioctl(fd, SNDCTL_DSP_STEREO, &stereo);
-	efrequency = frequency;
-	ioctl(fd, SNDCTL_DSP_SPEED, &efrequency);
+	/*
+	 * Set the stream format.  This ioctl() might fail, but should
+	 * return a format that works if it does.
+	 */
+	ioctl(fd, SNDCTL_DSP_SETFMT, &output.format.oss);
+	if (ioctl(fd, SNDCTL_DSP_SETFMT, &output.format.oss) == -1)
+		g_warning("SNDCTL_DSP_SETFMT ioctl failed: %s",
+			  strerror(errno));
 
-	if(ioctl(fd, SNDCTL_DSP_GETBLKSIZE, &blk_size) == -1)
+	stereo = output.channels - 1;
+	ioctl(fd, SNDCTL_DSP_STEREO, &stereo);
+	output.channels = stereo + 1;
+
+	oss_stereo_convert_func = oss_get_stereo_convert_func(output.channels,
+							      effect.channels);
+
+	if (ioctl(fd, SNDCTL_DSP_SPEED, &output.frequency) == -1)
+		g_warning("SNDCTL_DSP_SPEED ioctl failed: %s", strerror(errno));
+
+	if (ioctl(fd, SNDCTL_DSP_GETBLKSIZE, &blk_size) == -1)
 		blk_size = 1L << fragsize;
 
-	ebps = efrequency * channels;
-	if (oss_format == AFMT_U16_BE || oss_format == AFMT_U16_LE ||
-	    oss_format == AFMT_S16_BE || oss_format == AFMT_S16_LE)
-		ebps *= 2;
+	oss_convert_func =
+		oss_get_convert_func(output.format.oss,
+				     oss_get_format(effect.format.xmms));
 
-	/* Stupid hack to find out if the driver support selects,
-	   some drivers won't work properly without a select and
-	   some won't work with a select :/ */
+	/*
+	 * Stupid hack to find out if the driver support selects, some
+	 * drivers won't work properly without a select and some won't
+	 * work with a select :/
+	 */
 	
 	tv.tv_sec = 0;
 	tv.tv_usec = 50000;
 	FD_ZERO(&set);
 	FD_SET(fd, &set);
 	ret = select(fd + 1, NULL, &set, NULL, &tv);
-	if(ret > 0)
+	if (ret > 0)
 		select_works = TRUE;
 	else
 		select_works = FALSE;
@@ -583,7 +655,7 @@ void oss_set_audio_params(void)
 
 gint oss_open(AFormat fmt, gint rate, gint nch)
 {
-	if (oss_cfg.use_alt_audio_device)
+	if (oss_cfg.use_alt_audio_device && oss_cfg.alt_audio_device)
 		device_name = g_strdup(oss_cfg.alt_audio_device);
 	else
 	{
@@ -601,18 +673,18 @@ gint oss_open(AFormat fmt, gint rate, gint nch)
 		return 0;
 	}
 
-	oss_setup_format(fmt,rate,nch);
-	
-	input_format = format;
-	input_channels = channels;
-	input_frequency = frequency;
-	input_bps = bps;
+	input.format.xmms = fmt;
+	input.frequency = rate;
+	input.channels = nch;
+	input.bps = oss_calc_bitrate(oss_get_format(fmt), rate, nch);
 
+	oss_setup_format(fmt, rate, nch);
+	
 	realtime = xmms_check_realtime_priority();
 	
 	if(!realtime)
 	{
-		buffer_size = (oss_cfg.buffer_size * input_bps) / 1000;
+		buffer_size = (oss_cfg.buffer_size * input.bps) / 1000;
 		if (buffer_size < 8192)
 			buffer_size = 8192;
 		prebuffer_size = (buffer_size * oss_cfg.prebuffer) / 100;
@@ -623,14 +695,13 @@ gint oss_open(AFormat fmt, gint rate, gint nch)
 		buffer = g_malloc0(buffer_size);
 	}
 	flush = -1;
-	prebuffer = 1;
+	prebuffer = TRUE;
 	wr_index = rd_index = output_time_offset = written = output_bytes = 0;
 	paused = FALSE;
 	do_pause = FALSE;
 	unpause = FALSE;
 	remove_prebuffer = FALSE;
 
-	oss_set_audio_params();
 	going = 1;
 	if (!realtime)
 		pthread_create(&buffer_thread, NULL, oss_loop, NULL);
