@@ -24,4 +24,9 @@ void show_prefs_window(void);
 void prefswin_vplugins_rescan(void);
 void prefswin_show_vis_plugins_page(void);
 
+struct option_info {
+	GtkWidget * button;
+	gboolean * cfg;
+};
+
 #endif
